import os, stat, errno, subprocess, socket, platform
import logging
import time
from px_server.helper import download_and_extract_px_server_package

logging.getLogger().setLevel(logging.INFO)

class Server:

    port = None

    process = None

    px_server_executable_file_path = None

    def __init__(self):
        self.setup()
        self.start()

    def __del__(self):
        self.stop()

    def setup(self):
        self.px_server_executable_file_path = download_and_extract_px_server_package()

    def start(self):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind(("",0))
        self.port = s.getsockname()[1]

        logging.info('Start px server from {} file on port {}'.format(self.px_server_executable_file_path, self.port))
        self.process = subprocess.Popen([self.px_server_executable_file_path, '--port', str(self.port)])
        time.sleep(1)

    def stop(self):
        if self.process is not None:
            logging.info('Stop px server')
            logging.info('Stop px server process {}'.format(self.process.pid))
            self.process.terminate()
            logging.info('px server stopped')



import grpc
import px.px_pb2 as px
from px.px_pb2_grpc import BrowserStub
from px_server.server import Server
import logging

logging.getLogger().setLevel(logging.INFO)

class Client:

    channel = None

    stub = None

    debug = False

    server = None

    url = None

    def __init__(self, url=None, debug=False):
        if url is None:
            self.server = Server()
            url = 'localhost:{}'.format(self.server.port)

        self.url = url
        self.channel = grpc.insecure_channel(self.url)
        grpc.channel_ready_future(self.channel).result(timeout=300)
        self.stub = BrowserStub(self.channel)
        self.debug = debug

    def __del__(self):
        if self.channel is not None:
            self.channel.close()
            logging.info('Channel closed')

    def do_request(self, actions):
        request = px.DoRequest(actions=[actions])
        self.stub.Do(request)

    def clear_and_type(self, selector, text):
        actions = px.Action(clearAndTypeAction=px.ClearAndTypeAction(selector=selector, text=text))
        self.do_request(actions)

    def click(self, selector, button=None, clickCount=None):
        actions = px.Action(clickAction=px.ClickAction(selector=selector, button=button, clickCount=clickCount))
        self.do_request(actions)

    def close(self):
        actions = px.Action(closeAction=px.CloseAction())
        self.do_request(actions)

    def cookies(self):
        actions = px.Action(cookiesAction=px.CookiesAction())
        self.do_request(actions)

    def delete_cookies(self):
        actions = px.Action(deleteCookiesAction=px.DeleteCookiesAction())
        self.do_request(actions)

    def get_text(self, selector):
        actions = px.Action(getInnerTextAction=px.GetInnerTextAction(selector=selector))
        self.do_request(actions)

    def goto(self, url):
        actions = px.Action(gotoAction=px.GotoAction(url=url))
        self.do_request(actions)

    def launch(self):
        actions = px.Action(launchAction=px.LaunchAction())
        self.do_request(actions)

    def select(self, selector, values):
        actions = px.Action(selectAction=px.SelectAction(selector=selector, values=values))
        self.do_request(actions)

    def type(self, selector, text):
        actions = px.Action(typeAction=px.TypeAction(selector=selector, text=text))
        self.do_request(actions)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from px import px_pb2 as px_dot_px__pb2


class BrowserStub(object):
    """Missing associated documentation comment in .proto file"""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Do = channel.unary_unary(
                '/px.Browser/Do',
                request_serializer=px_dot_px__pb2.DoRequest.SerializeToString,
                response_deserializer=px_dot_px__pb2.DoResponse.FromString,
                )


class BrowserServicer(object):
    """Missing associated documentation comment in .proto file"""

    def Do(self, request, context):
        """Missing associated documentation comment in .proto file"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_BrowserServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Do': grpc.unary_unary_rpc_method_handler(
                    servicer.Do,
                    request_deserializer=px_dot_px__pb2.DoRequest.FromString,
                    response_serializer=px_dot_px__pb2.DoResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'px.Browser', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Browser(object):
    """Missing associated documentation comment in .proto file"""

    @staticmethod
    def Do(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/px.Browser/Do',
            px_dot_px__pb2.DoRequest.SerializeToString,
            px_dot_px__pb2.DoResponse.FromString,
            options, channel_credentials,
            call_credentials, compression, wait_for_ready, timeout, metadata)
